package at.kugel.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
/**
 * Calculate and output statistics for the full build. To be called from ANT with Ant's Java task.
 * First version with only PMD and source lines.
 * @author KoR
 */
public class BuildStatisticsInitialVersion {

   private static long countLinesIn(String pFilename) throws IOException {
      final BufferedReader in = new BufferedReader(new FileReader(pFilename));
      String t;
      long count = 0;
      while ((t = in.readLine()) != null) {
         if (!t.trim().equals("")) {
            count++;
         }
      }
      in.close();
      return count;
   }

   private static long countLinesInDir(File pDir, FilenameFilter pFilter) throws IOException {
      if (!pDir.isDirectory()) {
         throw new IOException("not a dir: " + pDir);
      }

      long linesInFiles = 0;

      final File[] files = pDir.listFiles();
      for (int i = 0; i < files.length; i++) {
         if (files[i].isFile()) {
            if (pFilter.accept(pDir, files[i].getName())) {
               linesInFiles += countLinesIn(files[i].getPath());
            }
         } else {
            linesInFiles += countLinesInDir(files[i], pFilter);
         }
      }
      return linesInFiles;
   }

   public static long countJavaSourceLinesRecursive(String pInDir) throws IOException {
      final FilenameFilter filter = new FilenameFilter() {
         public boolean accept(File dir, String name) {
            return name.endsWith(".java");
         }
      };
      return countLinesInDir(new File(pInDir), filter);
   }

   public static long countPMDLinesIn(String pFilename) throws IOException {
      final String PMD_NO_ERROR_LINE = "No problems found!";

      final BufferedReader in = new BufferedReader(new FileReader(pFilename));
      String t;
      long count = 0;
      while ((t = in.readLine()) != null) {
         if (!t.trim().equals("") && !t.equals(PMD_NO_ERROR_LINE)) {
            count++;
         }
      }
      in.close();
      return count;
   }

   public static void main(String[] args) throws IOException {

      if (!new File(args[0]).exists()) {
         throw new IOException("dir " + args[0] + " does not exist");
      }

      long linesInFiles = countJavaSourceLinesRecursive(args[0]);

      int[] multiplier = { 1000, 100, 10, 1 }; // error, warn, info, format
      String[] pmdLogs = { args[1], args[2], args[3], args[4] };

      long totalErrors = 0;
      for (int i = 0; i < pmdLogs.length; i++) {
         totalErrors += countPMDLinesIn(pmdLogs[i]) * multiplier[i];
      }

      int relError = (int) (totalErrors * 100 / linesInFiles);
      System.out.println("Total Weighted Errors Percentage: " + relError + "%");
   }

}
